@@ -22,6 +22,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  *****************************************************************************/
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <unistd.h>
 #include <ctype.h>
@@ -1013,6 +1016,13 @@ static int find_helper_file(const char *basename, int (*cb)(const char *))
           }
      }
 
+#ifdef SYSCONFDIR
+     snprintf(fname, sizeof(fname), SYSCONFDIR "/%s", basename);
+     if (cb(fname)) 
+     {
+          return 1;
+     }
+#else
      snprintf(fname, sizeof(fname), "/etc/%s", basename);
      if (cb(fname)) 
      {
@@ -1024,6 +1034,7 @@ static int find_helper_file(const char *basename, int (*cb)(const char *))
      {
           return 1;
      }
+#endif
 
      snprintf(fname, sizeof(fname), "/usr/local/mozilla/%s", basename);
      if (cb(fname)) 
@@ -1336,27 +1347,28 @@ static void do_read_config(void)
  * Check URL is safe. Since href's are passed to an app as an argument, just 
  * check for ways that  a shell can be tricked into executing a command.
  *
- * @param[in] url The URL to check
+ * @param[in] name   The name to check
+ * @param[in] isURL  Is the name a URL or filename?
  *
  * @return 1(true) if OK, 0(false) if not
  *
  *****************************************************************************/
-static int safeURL(const char* url)
+static int safeName(const char* name, int isURL)
 {
      int  i = 0; 
-     const int len = strlen(url);
+     const int len = strlen(name);
     
-     if (url[0] == '/')
+     if ((name[0] == '/') && (isURL))
      {
-          D("safeURL() - reject URL as starts with '/'\n");
+          D("safeName() - reject URL '%s' as starts with '/'\n", name);
 	  return 0;
      }
 
      for (i = 0; i < len; i++)
      {
-	  if (url[i] == '`' || url[i] == ';')
+	  if (name[i] == '`' || name[i] == ';')
 	  {
-               D("safeURL() - reject URL as contains either ';' or '`'\n");
+               D("safeName() - reject '%s' as contains either ';' or '`'\n", name);
 	       /* Somebody's trying to do something naughty. */
 	       return 0;
 	  }
@@ -1949,6 +1961,9 @@ static const char * getPluginDescription(void)
      snprintf(desc_buffer, sizeof(desc_buffer),
 		   "MozPlugger version "
 		   VERSION
+#ifdef GCOV
+                   "(gcov)"
+#endif
 		   ", maintained by Louis Bavoil and Peter Leese, a fork of plugger"
                    " written by Fredrik H&uuml;binette.<br>"
 		   "For documentation on how to configure mozplugger, "
@@ -2089,8 +2104,13 @@ NPError NPP_GetValue(void *instance, NPPVariable variable, void *value)
 	  break;
 
      case NPPVpluginNeedsXEmbed:
-          D("NPP_GetValue(NPPVpluginNeedsXEmbed)\n"); 
+          D("NPP_GetValue(NPPVpluginNeedsXEmbed)\n");
+#ifdef ALWAYS_NEEDS_XEMBED 
+          /* For Chromium always return 1 */
+          *((NPBool *)value) = 1;
+#else
           *((NPBool *)value) = getPluginNeedsXembed(instance, &err);
+#endif
           break; 
 
      case NPPVpluginScriptableNPObject :
@@ -2391,11 +2411,6 @@ NPError NPP_New(NPMIMEType pluginType,
 
      if (url)
      {
-          if( !safeURL(url))
-          {
-	       return NPERR_GENERIC_ERROR;
-          }
-
           THIS->url = url;
 
           /* Mozilla does not support the following protocols directly and
@@ -2536,11 +2551,12 @@ NPError NPP_Destroy(NPP instance, NPSavedData** save)
  *
  * @param[in] THIS     Pointer to the plugin instance data
  * @param[in] fname    The filename of the embedded object
+ * @param[in] isURL    Is the filename a URL?
  *
  * @return Nothing
  *
  *****************************************************************************/
-static void new_child(NPP instance, const char* fname)
+static void new_child(NPP instance, const char* fname, int isURL)
 {
      int commsPipe[2];
      data_t * THIS;
@@ -2554,10 +2570,6 @@ static void new_child(NPP instance, const char* fname)
           return;
      }
 
-     if (instance == NULL)
-     {
-	  return;
-     }
 
      THIS = instance->pdata;
 
@@ -2574,6 +2586,13 @@ static void new_child(NPP instance, const char* fname)
           return;
      }
 
+     if(!safeName(fname, isURL) 
+             || (THIS->urlFragment && !safeName(THIS->urlFragment, 0)))
+     {
+	  NPN_Status(instance, "MozPlugger: Detected unsafe URL aborting!");
+	  return;
+     }
+
      if (socketpair(AF_UNIX, SOCK_STREAM, 0, commsPipe) < 0)
      {
 	  NPN_Status(instance, "MozPlugger: Failed to create a pipe!");
@@ -2647,6 +2666,9 @@ static void new_child(NPP instance, const char* fname)
 /**
  * Guess a temporary file name
  *
+ * Creates a temporary file name based on the fileName provided. Checks that
+ * the filename created does not include any dangereous or awkward characters.
+ *
  * @return file descriptor
  *
  *****************************************************************************/
@@ -2655,27 +2677,46 @@ static int guessTmpFile(const char * fileName, int soFar,
 {
      int i;
      int fd = -1;
+     const int spaceLeft = maxTmpFilePathLen - soFar;
 
      for(i = 0; i <= 100; i++)
      {
           if(i == 0)
 	  {
-	       strncpy(&tmpFilePath[soFar], fileName,
-				maxTmpFilePathLen - soFar);
+               /* Whilst creating a pdf watch out for characters that may
+                * cause issues... */
+
+               char * dst = &tmpFilePath[soFar];
+               const char * src = fileName;
+               int left = spaceLeft;
+               while(left > 0)
+               {
+                    char ch = *src++;
+                    if((ch == ';') || (ch == '`') || (ch == '&') ||
+                                      (ch == ' ') || (ch == '\t'))
+                    {
+                        ch = '_';
+                    }
+                    else if(ch == '\0')
+                    {
+                        break;
+                    }
+                    *dst++ = ch;
+               }
+	       strncpy(&tmpFilePath[soFar], fileName, spaceLeft);
           }
 	  else if (i == 100)
 	  {
-               strncpy(&tmpFilePath[soFar], "XXXXXX",
-                                                     maxTmpFilePathLen - soFar);
+               strncpy(&tmpFilePath[soFar], "XXXXXX", spaceLeft);
                fd = mkstemp(tmpFilePath);
 	       break;
 	  }
 	  else
 	  {
-	       snprintf(&tmpFilePath[soFar],
-				 maxTmpFilePathLen - soFar,
+	       snprintf(&tmpFilePath[soFar], spaceLeft,
 				 "%03i-%s", i, fileName);
 	  }
+          tmpFilePath[maxTmpFilePathLen-1] = '\0';
 
 	  fd = open(tmpFilePath, O_CREAT | O_EXCL | O_WRONLY,
 			  S_IRUSR | S_IWUSR);
@@ -2802,10 +2843,8 @@ NPError NPP_NewStream(NPP instance,
      }
 
      /*  Replace the stream's URL with the URL in THIS->href if it
-      *  exists.  Since we're going to be replacing <embed>'s src with
-      *  href, we want to make sure that the URL is not going to try
-      *  anything dirty. */
-     if((THIS->href != NULL) && (safeURL(THIS->href)))
+      *  exists. */ 
+     if(THIS->href != NULL)
      {
 	  D("Replacing SRC with HREF... \n");
 
@@ -2821,7 +2860,7 @@ NPError NPP_NewStream(NPP instance,
      {
           /* URL has changed */
           D("URL has changed to %s\n", stream->url);
-          THIS->url = stream->url;
+          THIS->url = (char *) stream->url;
           refind_command = 1;
      }
     
@@ -2890,7 +2929,7 @@ NPError NPP_NewStream(NPP instance,
      }
      else
      {
-         new_child(instance, THIS->url);
+         new_child(instance, THIS->url, 1);
      }
 
      *stype = NP_NORMAL;
@@ -2914,7 +2953,11 @@ void NPP_StreamAsFile(NPP instance,
 		      const char* fname)
 {
      D("NPP_StreamAsFile() - instance=%p\n", instance);
-     new_child(instance, fname);
+
+     if (instance != NULL)
+     {
+          new_child(instance, fname, 0);
+     }
 }
 
 /*****************************************************************************/
@@ -3032,7 +3075,7 @@ NPError NPP_SetWindow(NPP instance, NPWindow* window)
           /* Extract from the URL the various additional information */
           parseURL(THIS, THIS->url, 0, 0);
 
-	  new_child(instance, THIS->url);
+	  new_child(instance, THIS->url, 1);
           THIS->url = NULL;                 /* Stops new_child from being called again */
 	  return NPERR_NO_ERROR;
      }
@@ -3075,28 +3118,31 @@ NPError NPP_SetWindow(NPP instance, NPWindow* window)
 NPError NPP_DestroyStream(NPP instance, NPStream *stream, NPError reason)
 {
      D("NPP_DestroyStream() - instance=%p\n", instance);
-     if(instance)
+     
+     if (!instance)
      {
-          data_t * const THIS = instance->pdata;
-          
-          if(THIS->tmpFileFd >= 0)
-          {
-               close(THIS->tmpFileFd);
-               THIS->tmpFileFd = -1;
+	  return NPERR_INVALID_INSTANCE_ERROR;
+     }
 
-               if( THIS->tmpFileName != NULL)
+     data_t * const THIS = instance->pdata;
+          
+     if(THIS->tmpFileFd >= 0)
+     {
+          close(THIS->tmpFileFd);
+          THIS->tmpFileFd = -1;
+
+          if( THIS->tmpFileName != NULL)
+          {
+               D("Closing Temporary file \'%s\'\n", THIS->tmpFileName);     
+               if(THIS->commsPipeFd < 0)
                {
-                    D("Closing Temporary file \'%s\'\n", THIS->tmpFileName);     
-                    if(THIS->commsPipeFd < 0)
-                    {
-                         new_child(instance, THIS->tmpFileName);
-                    }
-               }
-               else
-               {
-                   D("Closing stdin pipe\n");
+                    new_child(instance, THIS->tmpFileName, 0);
                }
           }
+          else
+          {
+              D("Closing stdin pipe\n");
+          }
      }
      return NPERR_NO_ERROR;
 }
@@ -3233,14 +3279,14 @@ int32_t NPP_WriteReady(NPP instance, NPStream *stream)
                D("Nothing to do - Application will handle stream\n");
                /* STREAM, close NPAPI stream and pass URL to application */
           }
-     }
 
-     if(size == 0)
-     {
-	  /* Tell the browser that it can finish with the stream
-   	     (actually we just wanted the name of the stream!)
-	     And not the stream data. */
-          NPN_DestroyStream(instance, stream, NPRES_DONE);
+          if(size == 0)
+          {
+	       /* Tell the browser that it can finish with the stream
+   	          (actually we just wanted the name of the stream!)
+	          And not the stream data. */
+               NPN_DestroyStream(instance, stream, NPRES_DONE);
+          }
      }
      return size;
 }
