@@ -47,6 +47,9 @@
  * Peter Leese - modified for mozplugger
  *
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"         /* For VERSION */
+#endif
 
 #define XP_UNIX 1
 
